#include "dialogs.h"

#include <kapp.h>
#include <klocale.h>
#include <kconfig.h>

#include "defines.h"
#include "version.h"

QString HSString(const char *str, uint i)
{
	return QString("%1%2").arg(str).arg(i);
}

#define HSName(i)  HSString(HS_NAME, i)
#define HSScore(i) HSString(HS_SCORE, i)
#define HSLevel(i) HSString(HS_LEVEL, i)

#define BORDER  10
#define TBORDER  5
#define PS_INC   4

WHighScores::WHighScores(QWidget *parent, const GameOverItem *god,
						 bool *res)
: KDialog(parent, 0, TRUE)
{
	KConfig *conf = kapp->config();
	conf->setGroup(HS_GRP);

	// find the number of score entries and the position of
	// the new score (if set)
	// (we assume they are ordered and valid ...)
	k = NB_HS;
	uint s, i;
	for (i=0; i<NB_HS; i++) {
		s = conf->readNumEntry(HSScore(i), 0);
		if ( s==0 ) { // end of entries
				if ( k==NB_HS ) k = i;
			break;
		}
		if ( god && s<god->score && k==NB_HS ) k = i;
	}
	
	if (god) {
		if (res) *res = ( k!=NB_HS );
		if ( k==NB_HS ) return; // not a better score

		// insert the new score
		i++;
		for (uint j=i-1; j>k; j--) {
			conf->writeEntry(HSScore(j), conf->readEntry(HSScore(j-1)));
            conf->writeEntry(HSName(j), conf->readEntry(HSName(j-1)));
            conf->writeEntry(HSLevel(j), conf->readEntry(HSLevel(j-1)));
		}
		
		conf->writeEntry(HSScore(k), (uint)god->score);
		conf->writeEntry(HSName(k), i18n("Anonymous")); // default name
		conf->writeEntry(HSLevel(k), (uint)god->level);
	}
	
	QLabel *lab;
	QString str = i18n("Hall of Fame");
	setCaption(i18n("%1 : %2").arg(KSIRTET_NAME).arg(str));
	
/* top layout */
	QVBoxLayout *top = new QVBoxLayout(this, BORDER);
	top->setResizeMode( QLayout::Fixed );
	
/* title */
	lab = new QLabel(str, this);
	QFont f( lab->font() );
	QFontInfo fi(f);
	f.setPointSize(fi.pointSize()+PS_INC);
	f.setBold(TRUE);
	lab->setFont(f);
	lab->setAlignment(AlignCenter);
	lab->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	lab->setFixedSize( lab->sizeHint() + QSize(2*TBORDER, TBORDER) );
	lab->setBackgroundMode( QWidget::PaletteMidlight );
	top->addWidget(lab);
	top->addSpacing(2*BORDER); // some additional space
	top->addStretch(1);

	f = font();
	f.setBold(TRUE);

/* grid layout */
	if ( i==0 ) {
		lab = new QLabel(i18n("no score entry"), this);
		lab->setMinimumSize( lab->sizeHint() );
		lab->setFont(f);
		top->addWidget(lab);
	} else {
		gl = new QGridLayout(top, 4, i+1, 2*BORDER);
		
		lab = new QLabel(i18n("Rank"), this);
		lab->setMinimumSize( lab->sizeHint() );
		gl->addWidget(lab, 0, 0);
		
		lab = new QLabel(i18n("Player Name"), this);
		lab->setMinimumSize( lab->sizeHint() );
		gl->addWidget(lab, 0, 1);
		
		lab = new QLabel(i18n("Level"), this);
		lab->setMinimumSize( lab->sizeHint() );
		lab->setAlignment(AlignCenter);
		gl->addWidget(lab, 0, 2);
		
		lab = new QLabel(i18n("Score"), this);
		lab->setMinimumSize( lab->sizeHint() );
		lab->setAlignment(AlignRight);
		gl->addWidget(lab, 0, 3);
	}
	
	for (uint j=0; j<i; j++) {
		/* rank */
		str.sprintf("%i", j+1);
		lab = new QLabel(str, this);
		lab->setMinimumSize( lab->sizeHint() );
		gl->addWidget(lab, j+1, 0);

		/* name */
		if ( !god || (k!=j) ) {
			QString name = conf->readEntry(HSName(j));
			lab = new QLabel(this);
			lab->setFont(f);
			lab->setText(name);
			lab->setMinimumSize( lab->sizeHint() );
			gl->addWidget(lab, j+1, 1);
		} else {
			qle = new QLineEdit(this);
			qle->setMaxLength(10);
			qle->setFont(f);
			qle->setMinimumSize(qle->fontMetrics().maxWidth()*10,
								qle->sizeHint().height());
			qle->setFocus();
			connect(qle, SIGNAL(returnPressed()), SLOT(writeName()));
			gl->addWidget(qle, j+1, 1);
		}
		
		/* level */
		lab = new QLabel(conf->readEntry(HSLevel(j)), this);
		lab->setMinimumSize( lab->sizeHint() );
		lab->setAlignment(AlignCenter);
		gl->addWidget(lab, j+1, 2);

		/* score */
		lab = new QLabel(conf->readEntry(HSScore(j)), this);
		lab->setMinimumSize( lab->sizeHint() );
		lab->setFont(f);
		lab->setAlignment(AlignRight);
		gl->addWidget(lab ,j+1, 3);
	}

/* close button */
	pb = new QPushButton(i18n("Close"), this);
	pb->setFixedSize( pb->sizeHint() );
	connect(pb, SIGNAL(clicked()), SLOT(accept()));
	top->addSpacing(2*BORDER);
	top->addWidget(pb);
	if ( !god ) pb->setFocus();
	else pb->setEnabled(FALSE);

	exec();
}

void WHighScores::writeName()
{
	KConfig *conf = kapp->config();
	conf->setGroup(HS_GRP);
	QString str = qle->text();
	if ( str.length() ) conf->writeEntry(HSName(k), str);
	
	/* show the entered highscore */
	str = conf->readEntry(HSName(k));
	qle->setText(str);
	qle->setEnabled(FALSE);
	qle->clearFocus();
	qle->setMinimumSize( qle->fontMetrics().boundingRect(str).width(),
						 qle->sizeHint().height());

	pb->setEnabled(TRUE);
	//pb->setFocus(); //if I set the focus : it closes the dialogs ?!
}
