#include "gpiece.h"

//-----------------------------------------------------------------------------
uint GPieceInfo::maxWidth() const
{
	int min, max;
	uint w = 0;
	for (uint i=0; i<nbForms(); i++) {
		min = max = x(i)[0];
		for (uint k=0; k<nbBlocks(); k++) {
			int tmp = x(i)[k];
			if ( tmp>max ) max = tmp;
			else if ( tmp<min ) min = tmp;
		}
		w = (uint)QMAX(w, (uint)max-min);	
	}
	return w;
}

uint GPieceInfo::maxHeight() const
{
	int min, max;
	uint h = 0;
	for (uint i=0; i<nbForms(); i++) {
		min = max = y(i)[0];
		for (uint k=0; k<nbBlocks(); k++) {
			int tmp = y(i)[k];
			if ( tmp>max ) max = tmp;
			else if ( tmp<min ) min = tmp;
		}
		h = (uint)QMAX(h, (uint)max-min);	
	}
	return h;
}

//-----------------------------------------------------------------------------
void GPiece::rotate(bool left)
{
    for (uint i=0; i<blocks.size(); i++) {
        int tmp = blocks[i].x;
		blocks[i].x = blocks[i].y * (left ?  1 : -1);
		blocks[i].y = tmp         * (left ? -1 :  1);
    }
}

int GPiece::minX() const
{
    int min = blocks[0].x;
    for(uint i=1; i<blocks.size(); i++) min = QMIN(min, blocks[i].x);
    return min;
}

int GPiece::maxX() const
{
    int max = blocks[0].x;
    for(uint i=1; i<blocks.size(); i++) max = QMAX(max, blocks[i].x);
    return max;
}

int GPiece::minY() const
{
    int min = blocks[0].y;
    for(uint i=1; i<blocks.size(); i++) min = QMIN(min, blocks[i].y);
    return min;
}

int GPiece::maxY() const
{
    int max = blocks[0].y;
    for(uint i=1; i<blocks.size(); i++) max = QMAX(max, blocks[i].y);
    return max;
}

GPiece &GPiece::operator =(const GPiece &p)
{
	blocks = p.blocks.copy();
	type   = p.type;
	return *this;
}
