#ifndef TL_BOARD_H
#define TL_BOARD_H 

#include "board.h"
#include "tl_piece.h"

#define TL_BLOCK_WIDTH  15
#define TL_BLOCK_HEIGHT 15

/** Class that implements the management for the "Tetris Like" game */
class TLBoard : public Board
{
 public:
	TLBoard(bool graphical, GiftPool *, QWidget *parent, const char *name=0);
	
	void init(int seed);
	
 private:
	// standard methods
	void gluePiece();
	uint gift();
	void putGift(uint nb);
	void _putGift(uint nb);
	bool needRemoving();
	void remove();
	void clearLine(uint line);
	void copyLine(uint destLine, uint srcLine);
	GPieceInfo *createPieceInfo() const { return new TLPieceInfo; }
	GPiece *createPiece() const         { return new TLPiece; }
	
	// graphic methods
	uint blockWidth() const  { return TL_BLOCK_WIDTH; }
	uint blockHeight() const { return TL_BLOCK_HEIGHT; }
	void lightRemoved(bool light);
	const QColor *colors() const { return TL_COLORS; }
	void drawBlock(QPainter *, const QRect &, const QColor &,
				   bool remove) const;
	
	// AI methods
	bool think(const Board &, uint &nbRotations, int &dec);
	int thinkingPoints();
	Board *newThinkingBoard();

	QArray<uint> filled; // number of filled cases per line
	uint         _betterRotation, addRemoved;
	int          _betterPoints, _betterDec;
};

#endif
